@@ -92,14 +92,10 @@ print.hpaML <- function (x, ...)
 #' corresponding random variable is not conditioned. By default all
 #' \code{given} components are \code{NA} so unconditional marginal
 #' density will be plotted for the \code{ind}-th random variable.
-#' @template elipsis_Template
-plot.hpaML <- function (x, y = NULL, ..., ind = 1, given = NULL) 
+#' @template elipsis_plot_Template
+plot.hpaML <- function (x, y = NULL, ..., 
+                        ind = 1, given = NULL) 
 {
-  if (length(list(...)) > 0)
-  {
-    warning("Additional arguments passed through ... are ignored.")   
-  }
-
   if (!is.null(y))
   {
     warning("Note that y parameter currently ignored.")   
@@ -140,9 +136,42 @@ plot.hpaML <- function (x, y = NULL, ..., ind = 1, given = NULL)
   } else {
     h <- matrix(as.numeric(my_data[, ind]), ncol = 1)
   }
-
-  h_mean <- mean(h)
-  h_sd <- stats::sd(h)
+  
+  # Assign conditional and marginal indexes
+  given_ind_new <- !is.na(given)
+  
+  omit_ind_new <- rep(TRUE, data_col)
+  omit_ind_new[ind] <- FALSE
+  omit_ind_new[given_ind_new] <- FALSE
+  
+  d_ind <- (!given_ind_new) & (!omit_ind_new)
+  
+  # Estimate moments
+  if(is.null(given))
+  {
+    guven <- vector(mode = "numeric")
+  }
+  
+  h_mean <- ehpa(x = given,
+                 pol_coefficients = x$pol_coefficients, 
+                 pol_degrees = x$pol_degrees,
+                 given_ind = given_ind_new,
+                 omit_ind = omit_ind_new,
+                 mean = x$mean, sd = x$sd,
+                 is_validation = FALSE, 
+                 expectation_powers = d_ind)
+  h_mean2 <- ehpa(x = given,
+                  pol_coefficients = x$pol_coefficients, 
+                  pol_degrees = x$pol_degrees,
+                  given_ind = given_ind_new,
+                  omit_ind = omit_ind_new,
+                  mean = x$mean, sd = x$sd,
+                  is_validation = FALSE, 
+                  expectation_powers = d_ind * 2)
+  h_sd <- sqrt(h_mean2 - h_mean ^ 2)
+  print(c(h_mean, h_mean2, h_sd, d_ind))
+  
+  
 
   # Set maximum and minimum values for plots
   plot_min <- h_mean - 3.8 * h_sd;
@@ -150,12 +179,12 @@ plot.hpaML <- function (x, y = NULL, ..., ind = 1, given = NULL)
 
   if (!is.na(x$tr_left[ind]))
   {
-    plot_min <- max(c(x$tr_left[ind], plot_min));
+    plot_min <- max(c(x$tr_left[ind], min(h[, ind])))
   }
   
   if(!is.na(x$tr_right[ind]))
   {
-    plot_max <- min(c(x$tr_right[ind], plot_max));
+    plot_max <- min(c(x$tr_right[ind], max(h[, ind])))
   }
 
   # Generate values for plot
@@ -180,13 +209,6 @@ plot.hpaML <- function (x, y = NULL, ..., ind = 1, given = NULL)
     }
   }
 
-  # Assign conditional and marginal indexes
-  given_ind_new <- !is.na(given)
-  
-  omit_ind_new <- rep(TRUE, data_col)
-  omit_ind_new[ind] <- FALSE
-  omit_ind_new[given_ind_new] <- FALSE
-
   den <- NULL
   
   if(class(x_vec)[1] != "matrix")
@@ -213,13 +235,67 @@ plot.hpaML <- function (x, y = NULL, ..., ind = 1, given = NULL)
                   mean = x$mean, sd = x$sd,
                   tr_left = x$tr_left,
                   tr_right = x$tr_right,
-                  is_validation = FALSE)
+                  is_validation = TRUE)
   }
 
   # Plot the result
-  plot(x = x_vec[, ind], y = den,
-       xlim = c(plot_min, plot_max),
-       type = "l", lwd = 3,
-       main = "Random Errors Density Approximation Plot",
-       xlab = "value", ylab = "density");
+  
+    # prepare the arguments
+  plot_args <- list(...)
+  plot_args_names <- names(plot_args)
+  if(!("xlim" %in% plot_args_names))
+  {
+    plot_args$xlim <- c(plot_min, plot_max)
+  }
+  if(!("type" %in% plot_args_names))
+  {
+    plot_args$type <- "l"
+  }
+  if(!("lwd" %in% plot_args_names))
+  {
+    plot_args$lwd <- 3
+  }
+  if(!("main" %in% plot_args_names))
+  {
+    plot_args$main <- "Random Errors Density Approximation Plot"
+  }
+  if(!("xlab" %in% plot_args_names))
+  {
+    plot_args$xlab <- "value"
+  }
+  if(!("ylab" %in% plot_args_names))
+  {
+    plot_args$ylab <- "density"
+  }
+  plot_args$x = x_vec[, ind]
+  plot_args$y = den
+  
+    # make the plot
+  do.call(plot, plot_args)
 }
+
+#' Extract coefficients from hpaML object
+#' @param object Object of class "hpaML"
+#' @template elipsis_Template
+coef.hpaML <- function (object, ...)
+{
+  if (length(list(...)) > 0)
+  {
+    warning("Additional arguments passed through ... are ignored.")   
+  }
+  
+  return(object$x1)
+}
+
+#' Extract covariance matrix from hpaML object
+#' @param object Object of class "hpaML"
+#' @template elipsis_Template
+vcov.hpaML <- function (object, ...)
+{
+  if (length(list(...)) > 0)
+  {
+    warning("Additional arguments passed through ... are ignored.")   
+  }
+  
+  return(object$cov_mat)
+}
\ No newline at end of file
